/* Gina -- Virtual sticky notes
 * Copyright (C) 2006-2009  Andrea Bolognani <eof@kiyuko.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Homepage: http://www.kiyuko.org/software/gina
 */

#include <src/ginanote.h>
#include <glib/gstdio.h>
#include <gdk/gdk.h>




static char* string_substring (const char* self, glong offset, glong len);
struct _GinaNotePrivate {
	char* filename;
	gboolean changed;
	GtkWidget* menu_bar;
	GtkTextView* text_view;
};

#define GINA_NOTE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GINA_TYPE_NOTE, GinaNotePrivate))
enum  {
	GINA_NOTE_DUMMY_PROPERTY
};
static gint gina_note_count = 0;
static void gina_note_open (GinaNote* self);
static void gina_note_close (GinaNote* self);
static void gina_note_save (GinaNote* self);
static void gina_note_save_as (GinaNote* self);
static void __lambda0 (GinaNote* self);
static void ___lambda0_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda1 (GinaNote* self);
static void ___lambda1_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda2 (GinaNote* self);
static void ___lambda2_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda3 (GinaNote* self);
static void ___lambda3_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda4 (GinaNote* self);
static void ___lambda4_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda5 (GinaNote* self);
static void ___lambda5_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda6 (GinaNote* self);
static void ___lambda6_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda7 (GinaNote* self);
static void ___lambda7_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda8 (GinaNote* self);
static void ___lambda8_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda9 (GinaNote* self);
static void ___lambda9_gtk_toggle_action_toggled (GtkToggleAction* _sender, gpointer self);
static void __lambda10 (GinaNote* self);
static void ___lambda10_gtk_toggle_action_toggled (GtkToggleAction* _sender, gpointer self);
static void __lambda11 (GinaNote* self);
static void ___lambda11_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda13 (GtkAboutDialog* dialog, const char* link, GinaNote* self);
static void ___lambda13_gtk_about_dialog_activate_link_func (GtkAboutDialog* about, const char* link_, gpointer self);
static void __lambda12 (GinaNote* self);
static void ___lambda12_gtk_action_activate (GtkAction* _sender, gpointer self);
static void __lambda14 (GinaNote* self);
static void ___lambda14_gtk_text_buffer_changed (GtkTextBuffer* _sender, gpointer self);
static gboolean __lambda15 (GinaNote* widget, const GdkEvent* event, GinaNote* self);
static gboolean ___lambda15_gtk_widget_delete_event (GinaNote* _sender, const GdkEvent* event, gpointer self);
static GObject * gina_note_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gina_note_parent_class = NULL;
static void gina_note_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


GinaNote* gina_note_construct (GType object_type) {
	GinaNote * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


GinaNote* gina_note_new (void) {
	return gina_note_construct (GINA_TYPE_NOTE);
}


static void gina_note_open (GinaNote* self) {
	GError * inner_error;
	char* contents;
	gboolean success;
	GtkFileChooserDialog* open_dialog;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	contents = g_strdup ("");
	success = TRUE;
	open_dialog = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new ("Open...", (GtkWindow*) self, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL, NULL));
	/* Allow only one local file to be selected */
	gtk_file_chooser_set_local_only ((GtkFileChooser*) open_dialog, TRUE);
	gtk_file_chooser_set_select_multiple ((GtkFileChooser*) open_dialog, FALSE);
	do {
		gint response;
		response = gtk_dialog_run ((GtkDialog*) open_dialog);
		if (response == GTK_RESPONSE_OK) {
			{
				char* _tmp2;
				gboolean _tmp1;
				char* _tmp0;
				/* Get the contents of the selected file */
				_tmp2 = NULL;
				_tmp0 = NULL;
				_tmp1 = g_file_get_contents (gtk_file_chooser_get_filename ((GtkFileChooser*) open_dialog), &_tmp0, NULL, &inner_error);
				contents = (_tmp2 = _tmp0, contents = (g_free (contents), NULL), _tmp2);
				_tmp1;
				if (inner_error != NULL) {
					if (inner_error->domain == G_FILE_ERROR) {
						goto __catch0_g_file_error;
					}
					goto __finally0;
				}
				/* Make sure the file is encoded in UTF-8, since
				                         * it's the only encoding we know how to handle */
				if (!g_utf8_validate (contents, -1, NULL)) {
					success = FALSE;
				}
				/* Strip the trailing newline if present */
				if (g_str_has_suffix (contents, "\n")) {
					char* _tmp3;
					_tmp3 = NULL;
					contents = (_tmp3 = string_substring (contents, (glong) 0, g_utf8_strlen (contents, -1) - 1), contents = (g_free (contents), NULL), _tmp3);
				}
				success = TRUE;
			}
			goto __finally0;
			__catch0_g_file_error:
			{
				GError * e;
				e = inner_error;
				inner_error = NULL;
				{
					GtkMessageDialog* error_dialog;
					/* Show an error message, then let the user
					                         * choose another file */
					error_dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) open_dialog, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, e->message, NULL));
					gtk_dialog_run ((GtkDialog*) error_dialog);
					gtk_object_destroy ((GtkObject*) error_dialog);
					success = FALSE;
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					(error_dialog == NULL) ? NULL : (error_dialog = (g_object_unref (error_dialog), NULL));
				}
			}
			__finally0:
			if (inner_error != NULL) {
				contents = (g_free (contents), NULL);
				(open_dialog == NULL) ? NULL : (open_dialog = (g_object_unref (open_dialog), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
				g_clear_error (&inner_error);
				return;
			}
		} else {
			/* The user cancelled the operation */
			success = FALSE;
			break;
		}
	} while (!success);
	/* Change filename and set the text view's content */
	if (success) {
		char* _tmp5;
		const char* _tmp4;
		char* _tmp7;
		char* _tmp6;
		_tmp5 = NULL;
		_tmp4 = NULL;
		self->priv->filename = (_tmp5 = (_tmp4 = gtk_file_chooser_get_filename ((GtkFileChooser*) open_dialog), (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), self->priv->filename = (g_free (self->priv->filename), NULL), _tmp5);
		gtk_text_buffer_set_text (gtk_text_view_get_buffer (self->priv->text_view), contents, -1);
		self->priv->changed = FALSE;
		_tmp7 = NULL;
		_tmp6 = NULL;
		gtk_window_set_title ((GtkWindow*) self, _tmp7 = g_strconcat (_tmp6 = g_strconcat ("Gina (", self->priv->filename, NULL), ")", NULL));
		_tmp7 = (g_free (_tmp7), NULL);
		_tmp6 = (g_free (_tmp6), NULL);
	}
	gtk_object_destroy ((GtkObject*) open_dialog);
	contents = (g_free (contents), NULL);
	(open_dialog == NULL) ? NULL : (open_dialog = (g_object_unref (open_dialog), NULL));
}


static void gina_note_close (GinaNote* self) {
	g_return_if_fail (self != NULL);
	/* TODO
	             * Check if the note has been changed since
	             * loading/creation, and ask the user if he wants to save it */
	if (self->priv->changed) {
	}
	/*debug ("Save changes?");*/
	gtk_object_destroy ((GtkObject*) self);
	gina_note_count--;
	if (gina_note_count == 0) {
		gtk_main_quit ();
	}
}


static void gina_note_save (GinaNote* self) {
	GError * inner_error;
	char* contents;
	gboolean success;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	contents = NULL;
	success = TRUE;
	/* If the note has no file associated with it, the user must
	             * choose one */
	if (self->priv->filename == NULL) {
		gina_note_save_as (self);
		contents = (g_free (contents), NULL);
		return;
	}
	{
		GtkTextIter start = {0};
		GtkTextIter end = {0};
		char* _tmp1;
		const char* _tmp0;
		char* _tmp2;
		gtk_text_buffer_get_bounds (gtk_text_view_get_buffer (self->priv->text_view), &start, &end);
		_tmp1 = NULL;
		_tmp0 = NULL;
		contents = (_tmp1 = (_tmp0 = gtk_text_buffer_get_text (gtk_text_view_get_buffer (self->priv->text_view), &start, &end, FALSE), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), contents = (g_free (contents), NULL), _tmp1);
		/* Write to the selected file */
		_tmp2 = NULL;
		g_file_set_contents (self->priv->filename, _tmp2 = g_strconcat (contents, "\n", NULL), (glong) (-1), &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_FILE_ERROR) {
				goto __catch1_g_file_error;
			}
			goto __finally1;
		}
		_tmp2 = (g_free (_tmp2), NULL);
	}
	goto __finally1;
	__catch1_g_file_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			GtkMessageDialog* error_dialog;
			error_dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) self, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, e->message, NULL));
			gtk_dialog_run ((GtkDialog*) error_dialog);
			gtk_object_destroy ((GtkObject*) error_dialog);
			success = FALSE;
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			(error_dialog == NULL) ? NULL : (error_dialog = (g_object_unref (error_dialog), NULL));
		}
	}
	__finally1:
	if (inner_error != NULL) {
		contents = (g_free (contents), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
	if (success) {
		char* _tmp4;
		char* _tmp3;
		self->priv->changed = FALSE;
		_tmp4 = NULL;
		_tmp3 = NULL;
		gtk_window_set_title ((GtkWindow*) self, _tmp4 = g_strconcat (_tmp3 = g_strconcat ("Gina (", self->priv->filename, NULL), ")", NULL));
		_tmp4 = (g_free (_tmp4), NULL);
		_tmp3 = (g_free (_tmp3), NULL);
	}
	contents = (g_free (contents), NULL);
}


static void gina_note_save_as (GinaNote* self) {
	GError * inner_error;
	char* contents;
	gboolean success;
	GtkFileChooserDialog* save_as_dialog;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	contents = NULL;
	success = TRUE;
	save_as_dialog = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new ("Save as...", (GtkWindow*) self, GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL, NULL));
	/* Allow only one local file to be selected */
	gtk_file_chooser_set_local_only ((GtkFileChooser*) save_as_dialog, TRUE);
	gtk_file_chooser_set_select_multiple ((GtkFileChooser*) save_as_dialog, FALSE);
	gtk_file_chooser_set_do_overwrite_confirmation ((GtkFileChooser*) save_as_dialog, TRUE);
	do {
		gint response;
		response = gtk_dialog_run ((GtkDialog*) save_as_dialog);
		if (response == GTK_RESPONSE_OK) {
			{
				GtkTextIter start = {0};
				GtkTextIter end = {0};
				char* _tmp1;
				const char* _tmp0;
				char* _tmp2;
				gtk_text_buffer_get_bounds (gtk_text_view_get_buffer (self->priv->text_view), &start, &end);
				_tmp1 = NULL;
				_tmp0 = NULL;
				contents = (_tmp1 = (_tmp0 = gtk_text_buffer_get_text (gtk_text_view_get_buffer (self->priv->text_view), &start, &end, FALSE), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0)), contents = (g_free (contents), NULL), _tmp1);
				/* Write to the selected file */
				_tmp2 = NULL;
				g_file_set_contents (gtk_file_chooser_get_filename ((GtkFileChooser*) save_as_dialog), _tmp2 = g_strconcat (contents, "\n", NULL), (glong) (-1), &inner_error);
				if (inner_error != NULL) {
					if (inner_error->domain == G_FILE_ERROR) {
						goto __catch2_g_file_error;
					}
					goto __finally2;
				}
				_tmp2 = (g_free (_tmp2), NULL);
				success = TRUE;
			}
			goto __finally2;
			__catch2_g_file_error:
			{
				GError * e;
				e = inner_error;
				inner_error = NULL;
				{
					GtkMessageDialog* error_dialog;
					/* Show an error message, then let the user
					                         * choose another file */
					error_dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) save_as_dialog, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, e->message, NULL));
					gtk_dialog_run ((GtkDialog*) error_dialog);
					gtk_object_destroy ((GtkObject*) error_dialog);
					success = FALSE;
					(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					(error_dialog == NULL) ? NULL : (error_dialog = (g_object_unref (error_dialog), NULL));
				}
			}
			__finally2:
			if (inner_error != NULL) {
				contents = (g_free (contents), NULL);
				(save_as_dialog == NULL) ? NULL : (save_as_dialog = (g_object_unref (save_as_dialog), NULL));
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
				g_clear_error (&inner_error);
				return;
			}
		} else {
			/* The user cancelled the operation */
			success = FALSE;
			break;
		}
	} while (!success);
	/* Change filename and set the text view's content */
	if (success) {
		char* _tmp4;
		const char* _tmp3;
		char* _tmp6;
		char* _tmp5;
		_tmp4 = NULL;
		_tmp3 = NULL;
		self->priv->filename = (_tmp4 = (_tmp3 = gtk_file_chooser_get_filename ((GtkFileChooser*) save_as_dialog), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3)), self->priv->filename = (g_free (self->priv->filename), NULL), _tmp4);
		self->priv->changed = FALSE;
		_tmp6 = NULL;
		_tmp5 = NULL;
		gtk_window_set_title ((GtkWindow*) self, _tmp6 = g_strconcat (_tmp5 = g_strconcat ("Gina (", self->priv->filename, NULL), ")", NULL));
		_tmp6 = (g_free (_tmp6), NULL);
		_tmp5 = (g_free (_tmp5), NULL);
	}
	gtk_object_destroy ((GtkObject*) save_as_dialog);
	contents = (g_free (contents), NULL);
	(save_as_dialog == NULL) ? NULL : (save_as_dialog = (g_object_unref (save_as_dialog), NULL));
}


gint gina_note_main (char** args, int args_length1) {
	GinaNote* _tmp0;
	gtk_init (&args_length1, &args);
	_tmp0 = NULL;
	_tmp0 = g_object_ref_sink (gina_note_new ());
	(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	gtk_main ();
	return 0;
}


int main (int argc, char ** argv) {
	g_type_init ();
	return gina_note_main (argv, argc);
}


static void __lambda0 (GinaNote* self) {
	GinaNote* _tmp0;
	_tmp0 = NULL;
	_tmp0 = g_object_ref_sink (gina_note_new ());
	(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
}


static void ___lambda0_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda0 (self);
}


static void __lambda1 (GinaNote* self) {
	gina_note_open (self);
}


static void ___lambda1_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda1 (self);
}


static void __lambda2 (GinaNote* self) {
	gina_note_save (self);
}


static void ___lambda2_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda2 (self);
}


static void __lambda3 (GinaNote* self) {
	gina_note_save_as (self);
}


static void ___lambda3_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda3 (self);
}


static void __lambda4 (GinaNote* self) {
	gtk_main_quit ();
}


static void ___lambda4_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda4 (self);
}


static void __lambda5 (GinaNote* self) {
	g_signal_emit_by_name (self->priv->text_view, "cut-clipboard");
}


static void ___lambda5_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda5 (self);
}


static void __lambda6 (GinaNote* self) {
	g_signal_emit_by_name (self->priv->text_view, "copy-clipboard");
}


static void ___lambda6_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda6 (self);
}


static void __lambda7 (GinaNote* self) {
	g_signal_emit_by_name (self->priv->text_view, "paste-clipboard");
}


static void ___lambda7_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda7 (self);
}


static void __lambda8 (GinaNote* self) {
	g_signal_emit_by_name (self->priv->text_view, "select-all", TRUE);
}


static void ___lambda8_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda8 (self);
}


static void __lambda9 (GinaNote* self) {
	gboolean _tmp0;
	if ((g_object_get (self->priv->menu_bar, "visible", &_tmp0, NULL), _tmp0)) {
		gtk_widget_hide (self->priv->menu_bar);
	} else {
		gtk_widget_show_all (self->priv->menu_bar);
	}
}


static void ___lambda9_gtk_toggle_action_toggled (GtkToggleAction* _sender, gpointer self) {
	__lambda9 (self);
}


static void __lambda10 (GinaNote* self) {
	if (gtk_window_get_decorated ((GtkWindow*) self)) {
		gtk_window_set_decorated ((GtkWindow*) self, FALSE);
	} else {
		gtk_window_set_decorated ((GtkWindow*) self, TRUE);
	}
}


static void ___lambda10_gtk_toggle_action_toggled (GtkToggleAction* _sender, gpointer self) {
	__lambda10 (self);
}


static void __lambda11 (GinaNote* self) {
	GtkDialog* dialog;
	GtkLabel* label;
	dialog = g_object_ref_sink ((GtkDialog*) gtk_dialog_new_with_buttons ("Key bindings", (GtkWindow*) self, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CLOSE, GTK_RESPONSE_OK, NULL, NULL));
	gtk_dialog_set_has_separator (dialog, FALSE);
	gtk_container_set_border_width ((GtkContainer*) dialog, (guint) 5);
	label = g_object_ref_sink ((GtkLabel*) gtk_label_new (""));
	gtk_label_set_markup (label, "<b>Manage notes</b>\n" "Ctrl+N: Create new note\n" "Ctrl+O: Load note from file\n" "Ctrl+S: Save note\n" "Shift+Ctrl+S: Save note as...\n" "\n" "<b>Edit</b>\n" "Ctrl+C: Copy hilighted text\n" "Ctrl+X: Cut text\n" "Ctrl+V: Paste clipboard's contents\n" "Ctrl+A: Select all\n" "\n" "<b>Miscellaneous</b>\n" "Ctrl+M: Toggle menubar\n" "Ctrl+D: Toggle window decorations\n" "Ctrl+H: Show this help");
	gtk_container_add ((GtkContainer*) dialog->vbox, (GtkWidget*) label);
	gtk_widget_show_all ((GtkWidget*) dialog);
	gtk_dialog_run (dialog);
	gtk_object_destroy ((GtkObject*) dialog);
	(dialog == NULL) ? NULL : (dialog = (g_object_unref (dialog), NULL));
	(label == NULL) ? NULL : (label = (g_object_unref (label), NULL));
}


static void ___lambda11_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda11 (self);
}


static void __lambda13 (GtkAboutDialog* dialog, const char* link, GinaNote* self) {
	GError * inner_error;
	g_return_if_fail (dialog != NULL);
	g_return_if_fail (link != NULL);
	inner_error = NULL;
	{
		gtk_show_uri (gtk_window_get_screen ((GtkWindow*) self), link, (guint32) GDK_CURRENT_TIME, &inner_error);
		if (inner_error != NULL) {
			goto __catch4_g_error;
			goto __finally4;
		}
	}
	goto __finally4;
	__catch4_g_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_warning ("ginanote.vala:240: Couldn't open link: %s", e->message);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally4:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return;
	}
}


static void ___lambda13_gtk_about_dialog_activate_link_func (GtkAboutDialog* about, const char* link_, gpointer self) {
	__lambda13 (about, link_, self);
}


static void __lambda12 (GinaNote* self) {
	gint _tmp1_length1;
	char** _tmp1;
	char** _tmp0;
	gtk_about_dialog_set_url_hook (___lambda13_gtk_about_dialog_activate_link_func, self, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	gtk_show_about_dialog ((GtkWindow*) self, "title", "About Gina", "program-name", "Gina", "version", "1.0.0", "website", "http://www.kiyuko.org/software/gina", "comments", "Virtual sticky notes", "copyright", "Copyright © 2006-2009 Andrea Bolognani", "authors", (_tmp1 = (_tmp0 = g_new0 (char*, 2 + 1), _tmp0[0] = g_strdup ("Andrea Bolognani <eof@kiyuko.org>"), _tmp0[1] = NULL, _tmp0), _tmp1_length1 = 2, _tmp1), NULL);
	_tmp1 = (_vala_array_free (_tmp1, _tmp1_length1, (GDestroyNotify) g_free), NULL);
}


static void ___lambda12_gtk_action_activate (GtkAction* _sender, gpointer self) {
	__lambda12 (self);
}


static void __lambda14 (GinaNote* self) {
	self->priv->changed = TRUE;
	if (self->priv->filename == NULL) {
		gtk_window_set_title ((GtkWindow*) self, "* Gina");
	} else {
		char* _tmp1;
		char* _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		gtk_window_set_title ((GtkWindow*) self, _tmp1 = g_strconcat (_tmp0 = g_strconcat ("* Gina (", self->priv->filename, NULL), ")", NULL));
		_tmp1 = (g_free (_tmp1), NULL);
		_tmp0 = (g_free (_tmp0), NULL);
	}
}


static void ___lambda14_gtk_text_buffer_changed (GtkTextBuffer* _sender, gpointer self) {
	__lambda14 (self);
}


static gboolean __lambda15 (GinaNote* widget, const GdkEvent* event, GinaNote* self) {
	g_return_val_if_fail (widget != NULL, FALSE);
	gina_note_close (self);
}


static gboolean ___lambda15_gtk_widget_delete_event (GinaNote* _sender, const GdkEvent* event, gpointer self) {
	return __lambda15 (_sender, event, self);
}


static GObject * gina_note_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GinaNoteClass * klass;
	GObjectClass * parent_class;
	GinaNote * self;
	GError * inner_error;
	klass = GINA_NOTE_CLASS (g_type_class_peek (GINA_TYPE_NOTE));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GINA_NOTE (obj);
	inner_error = NULL;
	{
		char* menu_tree;
		GtkAction* action;
		GtkToggleAction* toggle_action;
		GtkUIManager* menu;
		GtkActionGroup* file_group;
		GtkAction* _tmp0;
		GtkAction* _tmp1;
		GtkAction* _tmp2;
		GtkAction* _tmp3;
		GtkAction* _tmp4;
		GtkAction* _tmp5;
		GtkActionGroup* edit_group;
		GtkAction* _tmp6;
		GtkAction* _tmp7;
		GtkAction* _tmp8;
		GtkAction* _tmp9;
		GtkAction* _tmp10;
		GtkActionGroup* view_group;
		GtkAction* _tmp11;
		GtkToggleAction* _tmp12;
		GtkToggleAction* _tmp13;
		GtkActionGroup* help_group;
		GtkAction* _tmp14;
		GtkAction* _tmp15;
		GtkAction* _tmp16;
		GtkTextView* _tmp17;
		GdkColor color = {0};
		GtkWidget* _tmp19;
		GtkWidget* _tmp18;
		GtkVBox* box;
		menu_tree = g_strdup ("<ui>" "  <menubar name=\"Menu\">" "    <menu action=\"File\">" "      <menuitem action=\"New\" />" "      <menuitem action=\"Open\" />" "      <separator />" "      <menuitem action=\"Save\" />" "      <menuitem action=\"SaveAs\" />" "      <separator />" "      <menuitem action=\"Quit\" />" "    </menu>" "    <menu action=\"Edit\">" "      <menuitem action=\"Cut\" />" "      <menuitem action=\"Copy\" />" "      <menuitem action=\"Paste\" />" "      <separator />" "      <menuitem action=\"SelectAll\" />" "    </menu>" "    <menu action=\"View\">" "      <menuitem action=\"ShowMenuBar\" />" "      <menuitem action=\"ToggleDecorations\" />" "    </menu>" "    <menu action=\"Help\">" "      <menuitem action=\"KeyBindings\" />" "      <menuitem action=\"About\" />" "    </menu>" "  </menubar>" "</ui>");
		gtk_window_set_title ((GtkWindow*) self, "Gina");
		/* The window can't be resized under this size: it would be
		             * quite pointless to have a note on screen if it cannot
		             * display even a tiny amount of content */
		g_object_set ((GtkWidget*) self, "width-request", 70, NULL);
		g_object_set ((GtkWidget*) self, "height-request", 30, NULL);
		/* The window is this big by default */
		g_object_set ((GtkWindow*) self, "default-width", 200, NULL);
		g_object_set ((GtkWindow*) self, "default-height", 130, NULL);
		action = NULL;
		toggle_action = NULL;
		/* Load the menu definition */
		menu = gtk_ui_manager_new ();
		{
			gtk_ui_manager_add_ui_from_string (menu, menu_tree, (gssize) (-1), &inner_error);
			if (inner_error != NULL) {
				goto __catch3_g_error;
				goto __finally3;
			}
		}
		goto __finally3;
		__catch3_g_error:
		{
			GError * e;
			e = inner_error;
			inner_error = NULL;
			{
				g_error ("ginanote.vala:94: %s\n", e->message);
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally3:
		if (inner_error != NULL) {
			menu_tree = (g_free (menu_tree), NULL);
			(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
			(toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL));
			(menu == NULL) ? NULL : (menu = (g_object_unref (menu), NULL));
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		file_group = gtk_action_group_new ("File");
		_tmp0 = NULL;
		action = (_tmp0 = gtk_action_new ("File", "_File", NULL, NULL), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp0);
		gtk_action_group_add_action (file_group, action);
		_tmp1 = NULL;
		action = (_tmp1 = gtk_action_new ("New", NULL, NULL, GTK_STOCK_NEW), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp1);
		g_signal_connect (action, "activate", (GCallback) ___lambda0_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (file_group, action, "<Control>n");
		_tmp2 = NULL;
		action = (_tmp2 = gtk_action_new ("Open", NULL, NULL, GTK_STOCK_OPEN), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp2);
		g_signal_connect (action, "activate", (GCallback) ___lambda1_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (file_group, action, "<Control>o");
		_tmp3 = NULL;
		action = (_tmp3 = gtk_action_new ("Save", NULL, NULL, GTK_STOCK_SAVE), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp3);
		g_signal_connect (action, "activate", (GCallback) ___lambda2_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (file_group, action, "<Control>s");
		_tmp4 = NULL;
		action = (_tmp4 = gtk_action_new ("SaveAs", NULL, NULL, GTK_STOCK_SAVE_AS), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp4);
		g_signal_connect (action, "activate", (GCallback) ___lambda3_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (file_group, action, "<Control><Shift>s");
		_tmp5 = NULL;
		action = (_tmp5 = gtk_action_new ("Quit", NULL, NULL, GTK_STOCK_QUIT), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp5);
		g_signal_connect (action, "activate", (GCallback) ___lambda4_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (file_group, action, "<Control>q");
		edit_group = gtk_action_group_new ("Edit");
		_tmp6 = NULL;
		action = (_tmp6 = gtk_action_new ("Edit", "_Edit", NULL, NULL), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp6);
		gtk_action_group_add_action (edit_group, action);
		_tmp7 = NULL;
		action = (_tmp7 = gtk_action_new ("Cut", NULL, NULL, GTK_STOCK_CUT), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp7);
		g_signal_connect (action, "activate", (GCallback) ___lambda5_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (edit_group, action, "<Control>x");
		_tmp8 = NULL;
		action = (_tmp8 = gtk_action_new ("Copy", NULL, NULL, GTK_STOCK_COPY), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp8);
		g_signal_connect (action, "activate", (GCallback) ___lambda6_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (edit_group, action, "<Control>c");
		_tmp9 = NULL;
		action = (_tmp9 = gtk_action_new ("Paste", NULL, NULL, GTK_STOCK_PASTE), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp9);
		g_signal_connect (action, "activate", (GCallback) ___lambda7_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (edit_group, action, "<Control>v");
		_tmp10 = NULL;
		action = (_tmp10 = gtk_action_new ("SelectAll", NULL, NULL, GTK_STOCK_SELECT_ALL), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp10);
		g_signal_connect (action, "activate", (GCallback) ___lambda8_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (edit_group, action, "<Control>a");
		view_group = gtk_action_group_new ("View");
		_tmp11 = NULL;
		action = (_tmp11 = gtk_action_new ("View", "_View", NULL, NULL), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp11);
		gtk_action_group_add_action (view_group, action);
		_tmp12 = NULL;
		toggle_action = (_tmp12 = gtk_toggle_action_new ("ShowMenuBar", "Show _menubar", NULL, NULL), (toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL)), _tmp12);
		gtk_toggle_action_set_active (toggle_action, FALSE);
		g_signal_connect (toggle_action, "toggled", (GCallback) ___lambda9_gtk_toggle_action_toggled, self);
		gtk_action_group_add_action_with_accel (view_group, (GtkAction*) toggle_action, "<Control>m");
		_tmp13 = NULL;
		toggle_action = (_tmp13 = gtk_toggle_action_new ("ToggleDecorations", "Toggle _decorations", NULL, NULL), (toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL)), _tmp13);
		gtk_toggle_action_set_active (toggle_action, TRUE);
		g_signal_connect (toggle_action, "toggled", (GCallback) ___lambda10_gtk_toggle_action_toggled, self);
		gtk_action_group_add_action_with_accel (view_group, (GtkAction*) toggle_action, "<Control>d");
		help_group = gtk_action_group_new ("Help");
		_tmp14 = NULL;
		action = (_tmp14 = gtk_action_new ("Help", "_Help", NULL, NULL), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp14);
		gtk_action_group_add_action (help_group, action);
		_tmp15 = NULL;
		action = (_tmp15 = gtk_action_new ("KeyBindings", "Key _bindings", NULL, GTK_STOCK_HELP), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp15);
		g_signal_connect (action, "activate", (GCallback) ___lambda11_gtk_action_activate, self);
		gtk_action_group_add_action_with_accel (help_group, action, "<Control>h");
		_tmp16 = NULL;
		action = (_tmp16 = gtk_action_new ("About", NULL, NULL, GTK_STOCK_ABOUT), (action == NULL) ? NULL : (action = (g_object_unref (action), NULL)), _tmp16);
		g_signal_connect (action, "activate", (GCallback) ___lambda12_gtk_action_activate, self);
		gtk_action_group_add_action (help_group, action);
		gtk_ui_manager_insert_action_group (menu, file_group, 0);
		gtk_ui_manager_insert_action_group (menu, edit_group, 0);
		gtk_ui_manager_insert_action_group (menu, view_group, 0);
		gtk_ui_manager_insert_action_group (menu, help_group, 0);
		gtk_window_add_accel_group ((GtkWindow*) self, gtk_ui_manager_get_accel_group (menu));
		_tmp17 = NULL;
		self->priv->text_view = (_tmp17 = g_object_ref_sink ((GtkTextView*) gtk_text_view_new ()), (self->priv->text_view == NULL) ? NULL : (self->priv->text_view = (g_object_unref (self->priv->text_view), NULL)), _tmp17);
		g_signal_connect (gtk_text_view_get_buffer (self->priv->text_view), "changed", (GCallback) ___lambda14_gtk_text_buffer_changed, self);
		/* Set some properties of the text view */
		gtk_text_view_set_left_margin (self->priv->text_view, 3);
		gtk_text_view_set_right_margin (self->priv->text_view, 3);
		gtk_text_view_set_wrap_mode (self->priv->text_view, GTK_WRAP_WORD_CHAR);
		/* Set the background color */
		gdk_color_parse ("#f9f3a9", &color);
		gtk_widget_modify_base ((GtkWidget*) self->priv->text_view, GTK_STATE_NORMAL, &color);
		_tmp19 = NULL;
		_tmp18 = NULL;
		self->priv->menu_bar = (_tmp19 = (_tmp18 = gtk_ui_manager_get_widget (menu, "/Menu"), (_tmp18 == NULL) ? NULL : g_object_ref (_tmp18)), (self->priv->menu_bar == NULL) ? NULL : (self->priv->menu_bar = (g_object_unref (self->priv->menu_bar), NULL)), _tmp19);
		box = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0));
		gtk_box_pack_start ((GtkBox*) box, self->priv->menu_bar, FALSE, FALSE, (guint) 0);
		gtk_box_pack_start ((GtkBox*) box, (GtkWidget*) self->priv->text_view, TRUE, TRUE, (guint) 0);
		gtk_container_add ((GtkContainer*) self, (GtkWidget*) box);
		/* The menu bar is hidden by default */
		gtk_widget_show_all ((GtkWidget*) box);
		gtk_widget_hide (self->priv->menu_bar);
		g_signal_connect ((GtkWidget*) self, "delete-event", (GCallback) ___lambda15_gtk_widget_delete_event, self);
		/* XXX
		             * Would it be better not to show the note here, and to
		             * explicitly call show() when needed? */
		gtk_widget_show ((GtkWidget*) self);
		gina_note_count++;
		menu_tree = (g_free (menu_tree), NULL);
		(action == NULL) ? NULL : (action = (g_object_unref (action), NULL));
		(toggle_action == NULL) ? NULL : (toggle_action = (g_object_unref (toggle_action), NULL));
		(menu == NULL) ? NULL : (menu = (g_object_unref (menu), NULL));
		(file_group == NULL) ? NULL : (file_group = (g_object_unref (file_group), NULL));
		(edit_group == NULL) ? NULL : (edit_group = (g_object_unref (edit_group), NULL));
		(view_group == NULL) ? NULL : (view_group = (g_object_unref (view_group), NULL));
		(help_group == NULL) ? NULL : (help_group = (g_object_unref (help_group), NULL));
		(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
	}
	return obj;
}


static void gina_note_class_init (GinaNoteClass * klass) {
	gina_note_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GinaNotePrivate));
	G_OBJECT_CLASS (klass)->constructor = gina_note_constructor;
	G_OBJECT_CLASS (klass)->finalize = gina_note_finalize;
}


static void gina_note_instance_init (GinaNote * self) {
	self->priv = GINA_NOTE_GET_PRIVATE (self);
}


static void gina_note_finalize (GObject* obj) {
	GinaNote * self;
	self = GINA_NOTE (obj);
	self->priv->filename = (g_free (self->priv->filename), NULL);
	(self->priv->menu_bar == NULL) ? NULL : (self->priv->menu_bar = (g_object_unref (self->priv->menu_bar), NULL));
	(self->priv->text_view == NULL) ? NULL : (self->priv->text_view = (g_object_unref (self->priv->text_view), NULL));
	G_OBJECT_CLASS (gina_note_parent_class)->finalize (obj);
}


GType gina_note_get_type (void) {
	static GType gina_note_type_id = 0;
	if (gina_note_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GinaNoteClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gina_note_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GinaNote), 0, (GInstanceInitFunc) gina_note_instance_init, NULL };
		gina_note_type_id = g_type_register_static (GTK_TYPE_WINDOW, "GinaNote", &g_define_type_info, 0);
	}
	return gina_note_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
	g_free (array);
}




